@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011-2013 Raphaël Barrois
+# Copyright (c) 2011-2015 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,6 @@
 # THE SOFTWARE.
 
 import logging
-import warnings
 
 from . import containers
 from . import declarations
@@ -109,23 +108,6 @@ class FactoryMetaClass(type):
 
         attrs_meta = attrs.pop('Meta', None)
 
-        oldstyle_attrs = {}
-        converted_attrs = {}
-        for old_name, new_name in base_factory._OLDSTYLE_ATTRIBUTES.items():
-            if old_name in attrs:
-                oldstyle_attrs[old_name] = new_name
-                converted_attrs[new_name] = attrs.pop(old_name)
-        if oldstyle_attrs:
-            warnings.warn(
-                "Declaring any of %s at class-level is deprecated"
-                " and will be removed in the future. Please set them"
-                " as %s attributes of a 'class Meta' attribute." % (
-                    ', '.join(oldstyle_attrs.keys()),
-                    ', '.join(oldstyle_attrs.values()),
-                ),
-                PendingDeprecationWarning, 2)
-            attrs_meta = type('Meta', (object,), converted_attrs)
-
         base_meta = resolve_attribute('_meta', bases)
         options_class = resolve_attribute('_options_class', bases, FactoryOptions)
 
@@ -231,7 +213,7 @@ class FactoryOptions(object):
 
         self.counter_reference = self._get_counter_reference()
 
-        for parent in self.factory.__mro__[1:]:
+        for parent in reversed(self.factory.__mro__[1:]):
             if not hasattr(parent, '_meta'):
                 continue
             self.declarations.update(parent._meta.declarations)
@@ -322,14 +304,6 @@ class BaseFactory(object):
 
     _meta = FactoryOptions()
 
-    _OLDSTYLE_ATTRIBUTES = {
-        'FACTORY_FOR': 'model',
-        'ABSTRACT_FACTORY': 'abstract',
-        'FACTORY_STRATEGY': 'strategy',
-        'FACTORY_ARG_PARAMETERS': 'inline_args',
-        'FACTORY_HIDDEN_ARGS': 'exclude',
-    }
-
     # ID to use for the next 'declarations.Sequence' attribute.
     _counter = None
 
@@ -435,7 +409,7 @@ class BaseFactory(object):
                 retrieved DeclarationDict.
         """
         decls = cls._meta.declarations.copy()
-        decls.update(extra_defs)
+        decls.update(extra_defs or {})
         return decls
 
     @classmethod
@@ -709,7 +683,7 @@ class StubFactory(Factory):
 
     @classmethod
     def build(cls, **kwargs):
-        raise UnsupportedStrategy()
+        return cls.stub(**kwargs)
 
     @classmethod
     def create(cls, **kwargs):
