@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011-2013 Raphaël Barrois
+# Copyright (c) 2011-2015 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,6 @@
 
 
 import itertools
-import warnings
 import logging
 
 from . import compat
@@ -50,7 +49,7 @@ class OrderedDeclaration(object):
                 attributes
             containers (list of containers.LazyStub): The chain of SubFactory
                 which led to building this object.
-            create (bool): whether the target class should be 'built' or
+            create (bool): whether the model class should be 'built' or
                 'created'
             extra (DeclarationDict or None): extracted key/value extracted from
                 the attribute prefix
@@ -195,7 +194,7 @@ class Sequence(OrderedDeclaration):
         self.type = type
 
     def evaluate(self, sequence, obj, create, extra=None, containers=()):
-        logger.debug("Sequence: Computing next value of %r for seq=%d", self.function, sequence)
+        logger.debug("Sequence: Computing next value of %r for seq=%s", self.function, sequence)
         return self.function(self.type(sequence))
 
 
@@ -209,7 +208,7 @@ class LazyAttributeSequence(Sequence):
             of counter for the 'function' attribute.
     """
     def evaluate(self, sequence, obj, create, extra=None, containers=()):
-        logger.debug("LazyAttributeSequence: Computing next value of %r for seq=%d, obj=%r",
+        logger.debug("LazyAttributeSequence: Computing next value of %r for seq=%s, obj=%r",
                 self.function, sequence, obj)
         return self.function(obj, self.type(sequence))
 
@@ -434,7 +433,7 @@ class ExtractionContext(object):
 
 
 class PostGenerationDeclaration(object):
-    """Declarations to be called once the target object has been generated."""
+    """Declarations to be called once the model object has been generated."""
 
     def extract(self, name, attrs):
         """Extract relevant attributes from a dict.
@@ -502,14 +501,6 @@ class RelatedFactory(PostGenerationDeclaration):
 
     def __init__(self, factory, factory_related_name='', **defaults):
         super(RelatedFactory, self).__init__()
-        if factory_related_name == '' and defaults.get('name') is not None:
-            warnings.warn(
-                "Usage of RelatedFactory(SomeFactory, name='foo') is deprecated"
-                " and will be removed in the future. Please use the"
-                " RelatedFactory(SomeFactory, 'foo') or"
-                " RelatedFactory(SomeFactory, factory_related_name='foo')"
-                " syntax instead", PendingDeprecationWarning, 2)
-            factory_related_name = defaults.pop('name')
 
         self.name = factory_related_name
         self.defaults = defaults
