@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2013 Romain Command&
+# Copyright (c) 2015 Romain Command&
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -36,7 +36,8 @@ if sqlalchemy:
 else:
 
     class Fake(object):
-        FACTORY_SESSION = None
+        class Meta:
+            sqlalchemy_session = None
 
     models = Fake()
     models.StandardModel = Fake()
@@ -46,16 +47,18 @@ else:
 
 
 class StandardFactory(SQLAlchemyModelFactory):
-    FACTORY_FOR = models.StandardModel
-    FACTORY_SESSION = models.session
+    class Meta:
+        model = models.StandardModel
+        sqlalchemy_session = models.session
 
     id = factory.Sequence(lambda n: n)
     foo = factory.Sequence(lambda n: 'foo%d' % n)
 
 
 class NonIntegerPkFactory(SQLAlchemyModelFactory):
-    FACTORY_FOR = models.NonIntegerPk
-    FACTORY_SESSION = models.session
+    class Meta:
+        model = models.NonIntegerPk
+        sqlalchemy_session = models.session
 
     id = factory.Sequence(lambda n: 'foo%d' % n)
 
@@ -66,7 +69,7 @@ class SQLAlchemyPkSequenceTestCase(unittest.TestCase):
     def setUp(self):
         super(SQLAlchemyPkSequenceTestCase, self).setUp()
         StandardFactory.reset_sequence(1)
-        NonIntegerPkFactory.FACTORY_SESSION.rollback()
+        NonIntegerPkFactory._meta.sqlalchemy_session.rollback()
 
     def test_pk_first(self):
         std = StandardFactory.build()
@@ -85,18 +88,18 @@ class SQLAlchemyPkSequenceTestCase(unittest.TestCase):
 
         StandardFactory.reset_sequence()
         std2 = StandardFactory.create()
-        self.assertEqual('foo2', std2.foo)
-        self.assertEqual(2, std2.id)
+        self.assertEqual('foo0', std2.foo)
+        self.assertEqual(0, std2.id)
 
     def test_pk_force_value(self):
         std1 = StandardFactory.create(id=10)
-        self.assertEqual('foo1', std1.foo)  # sequence was set before pk
+        self.assertEqual('foo1', std1.foo)  # sequence and pk are unrelated
         self.assertEqual(10, std1.id)
 
         StandardFactory.reset_sequence()
         std2 = StandardFactory.create()
-        self.assertEqual('foo11', std2.foo)
-        self.assertEqual(11, std2.id)
+        self.assertEqual('foo0', std2.foo)  # Sequence doesn't care about pk
+        self.assertEqual(0, std2.id)
 
 
 @unittest.skipIf(sqlalchemy is None, "SQLalchemy not installed.")
@@ -104,26 +107,26 @@ class SQLAlchemyNonIntegerPkTestCase(unittest.TestCase):
     def setUp(self):
         super(SQLAlchemyNonIntegerPkTestCase, self).setUp()
         NonIntegerPkFactory.reset_sequence()
-        NonIntegerPkFactory.FACTORY_SESSION.rollback()
+        NonIntegerPkFactory._meta.sqlalchemy_session.rollback()
 
     def test_first(self):
         nonint = NonIntegerPkFactory.build()
-        self.assertEqual('foo1', nonint.id)
+        self.assertEqual('foo0', nonint.id)
 
     def test_many(self):
         nonint1 = NonIntegerPkFactory.build()
         nonint2 = NonIntegerPkFactory.build()
 
-        self.assertEqual('foo1', nonint1.id)
-        self.assertEqual('foo2', nonint2.id)
+        self.assertEqual('foo0', nonint1.id)
+        self.assertEqual('foo1', nonint2.id)
 
     def test_creation(self):
         nonint1 = NonIntegerPkFactory.create()
-        self.assertEqual('foo1', nonint1.id)
+        self.assertEqual('foo0', nonint1.id)
 
         NonIntegerPkFactory.reset_sequence()
         nonint2 = NonIntegerPkFactory.build()
-        self.assertEqual('foo1', nonint2.id)
+        self.assertEqual('foo0', nonint2.id)
 
     def test_force_pk(self):
         nonint1 = NonIntegerPkFactory.create(id='foo10')
@@ -131,4 +134,4 @@ class SQLAlchemyNonIntegerPkTestCase(unittest.TestCase):
 
         NonIntegerPkFactory.reset_sequence()
         nonint2 = NonIntegerPkFactory.create()
-        self.assertEqual('foo1', nonint2.id)
+        self.assertEqual('foo0', nonint2.id)
