@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011-2013 Raphaël Barrois
+# Copyright (c) 2011-2015 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -55,7 +55,7 @@ class FuzzyChoiceTestCase(unittest.TestCase):
 
         d = fuzzy.FuzzyChoice(options)
 
-        with mock.patch('random.choice', fake_choice):
+        with mock.patch('factory.fuzzy._random.choice', fake_choice):
             res = d.evaluate(2, None, False)
 
         self.assertEqual(6, res)
@@ -74,6 +74,24 @@ class FuzzyChoiceTestCase(unittest.TestCase):
         res = d.evaluate(2, None, False)
         self.assertIn(res, [0, 1, 2])
 
+    def test_lazy_generator(self):
+        class Gen(object):
+            def __init__(self, options):
+                self.options = options
+                self.unrolled = False
+
+            def __iter__(self):
+                self.unrolled = True
+                return iter(self.options)
+
+        opts = Gen([1, 2, 3])
+        d = fuzzy.FuzzyChoice(opts)
+        self.assertFalse(opts.unrolled)
+
+        res = d.evaluate(2, None, False)
+        self.assertIn(res, [1, 2, 3])
+        self.assertTrue(opts.unrolled)
+
 
 class FuzzyIntegerTestCase(unittest.TestCase):
     def test_definition(self):
@@ -89,24 +107,34 @@ class FuzzyIntegerTestCase(unittest.TestCase):
             self.assertIn(res, [0, 1, 2, 3, 4])
 
     def test_biased(self):
-        fake_randint = lambda low, high: low + high
+        fake_randrange = lambda low, high, step: (low + high) * step
 
         fuzz = fuzzy.FuzzyInteger(2, 8)
 
-        with mock.patch('random.randint', fake_randint):
+        with mock.patch('factory.fuzzy._random.randrange', fake_randrange):
             res = fuzz.evaluate(2, None, False)
 
-        self.assertEqual(10, res)
+        self.assertEqual((2 + 8 + 1) * 1, res)
 
     def test_biased_high_only(self):
-        fake_randint = lambda low, high: low + high
+        fake_randrange = lambda low, high, step: (low + high) * step
 
         fuzz = fuzzy.FuzzyInteger(8)
 
-        with mock.patch('random.randint', fake_randint):
+        with mock.patch('factory.fuzzy._random.randrange', fake_randrange):
             res = fuzz.evaluate(2, None, False)
 
-        self.assertEqual(8, res)
+        self.assertEqual((0 + 8 + 1) * 1, res)
+
+    def test_biased_with_step(self):
+        fake_randrange = lambda low, high, step: (low + high) * step
+
+        fuzz = fuzzy.FuzzyInteger(5, 8, 3)
+
+        with mock.patch('factory.fuzzy._random.randrange', fake_randrange):
+            res = fuzz.evaluate(2, None, False)
+
+        self.assertEqual((5 + 8 + 1) * 3, res)
 
 
 class FuzzyDecimalTestCase(unittest.TestCase):
@@ -136,7 +164,7 @@ class FuzzyDecimalTestCase(unittest.TestCase):
 
         fuzz = fuzzy.FuzzyDecimal(2.0, 8.0)
 
-        with mock.patch('random.uniform', fake_uniform):
+        with mock.patch('factory.fuzzy._random.uniform', fake_uniform):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(decimal.Decimal('10.0'), res)
@@ -146,7 +174,7 @@ class FuzzyDecimalTestCase(unittest.TestCase):
 
         fuzz = fuzzy.FuzzyDecimal(8.0)
 
-        with mock.patch('random.uniform', fake_uniform):
+        with mock.patch('factory.fuzzy._random.uniform', fake_uniform):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(decimal.Decimal('8.0'), res)
@@ -156,7 +184,7 @@ class FuzzyDecimalTestCase(unittest.TestCase):
 
         fuzz = fuzzy.FuzzyDecimal(8.0, precision=3)
 
-        with mock.patch('random.uniform', fake_uniform):
+        with mock.patch('factory.fuzzy._random.uniform', fake_uniform):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(decimal.Decimal('8.001').quantize(decimal.Decimal(10) ** -3), res)
@@ -204,7 +232,7 @@ class FuzzyDateTestCase(unittest.TestCase):
         fake_randint = lambda low, high: (low + high) // 2
         fuzz = fuzzy.FuzzyDate(self.jan1, self.jan31)
 
-        with mock.patch('random.randint', fake_randint):
+        with mock.patch('factory.fuzzy._random.randint', fake_randint):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(datetime.date(2013, 1, 16), res)
@@ -215,7 +243,7 @@ class FuzzyDateTestCase(unittest.TestCase):
             fuzz = fuzzy.FuzzyDate(self.jan1)
 
         fake_randint = lambda low, high: (low + high) // 2
-        with mock.patch('random.randint', fake_randint):
+        with mock.patch('factory.fuzzy._random.randint', fake_randint):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(datetime.date(2013, 1, 2), res)
@@ -322,7 +350,7 @@ class FuzzyNaiveDateTimeTestCase(unittest.TestCase):
         fake_randint = lambda low, high: (low + high) // 2
         fuzz = fuzzy.FuzzyNaiveDateTime(self.jan1, self.jan31)
 
-        with mock.patch('random.randint', fake_randint):
+        with mock.patch('factory.fuzzy._random.randint', fake_randint):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(datetime.datetime(2013, 1, 16), res)
@@ -333,7 +361,7 @@ class FuzzyNaiveDateTimeTestCase(unittest.TestCase):
             fuzz = fuzzy.FuzzyNaiveDateTime(self.jan1)
 
         fake_randint = lambda low, high: (low + high) // 2
-        with mock.patch('random.randint', fake_randint):
+        with mock.patch('factory.fuzzy._random.randint', fake_randint):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(datetime.datetime(2013, 1, 2), res)
@@ -440,7 +468,7 @@ class FuzzyDateTimeTestCase(unittest.TestCase):
         fake_randint = lambda low, high: (low + high) // 2
         fuzz = fuzzy.FuzzyDateTime(self.jan1, self.jan31)
 
-        with mock.patch('random.randint', fake_randint):
+        with mock.patch('factory.fuzzy._random.randint', fake_randint):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(datetime.datetime(2013, 1, 16, tzinfo=compat.UTC), res)
@@ -451,7 +479,7 @@ class FuzzyDateTimeTestCase(unittest.TestCase):
             fuzz = fuzzy.FuzzyDateTime(self.jan1)
 
         fake_randint = lambda low, high: (low + high) // 2
-        with mock.patch('random.randint', fake_randint):
+        with mock.patch('factory.fuzzy._random.randint', fake_randint):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual(datetime.datetime(2013, 1, 2, tzinfo=compat.UTC), res)
@@ -476,7 +504,7 @@ class FuzzyTextTestCase(unittest.TestCase):
 
         chars = ['a', 'b', 'c']
         fuzz = fuzzy.FuzzyText(prefix='pre', suffix='post', chars=chars, length=4)
-        with mock.patch('random.choice', fake_choice):
+        with mock.patch('factory.fuzzy._random.choice', fake_choice):
             res = fuzz.evaluate(2, None, False)
 
         self.assertEqual('preaaaapost', res)
@@ -494,3 +522,25 @@ class FuzzyTextTestCase(unittest.TestCase):
 
         for char in res:
             self.assertIn(char, ['a', 'b', 'c'])
+
+
+class FuzzyRandomTestCase(unittest.TestCase):
+    def test_seeding(self):
+        fuzz = fuzzy.FuzzyInteger(1, 1000)
+
+        fuzzy.reseed_random(42)
+        value = fuzz.evaluate(sequence=1, obj=None, create=False)
+
+        fuzzy.reseed_random(42)
+        value2 = fuzz.evaluate(sequence=1, obj=None, create=False)
+        self.assertEqual(value, value2)
+
+    def test_reset_state(self):
+        fuzz = fuzzy.FuzzyInteger(1, 1000)
+
+        state = fuzzy.get_random_state()
+        value = fuzz.evaluate(sequence=1, obj=None, create=False)
+
+        fuzzy.set_random_state(state)
+        value2 = fuzz.evaluate(sequence=1, obj=None, create=False)
+        self.assertEqual(value, value2)
