@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2011-2013 Raphaël Barrois
+# Copyright (c) 2011-2015 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -69,6 +69,9 @@ class FakeModel(object):
         def order_by(self, *args, **kwargs):
             return [1]
 
+        def using(self, db):
+            return self
+
     objects = FakeModelManager()
 
     def __init__(self, **kwargs):
@@ -766,6 +769,37 @@ class UsingFactoryTestCase(unittest.TestCase):
         test_object_alt = TestObjectFactory.build()
         self.assertEqual(None, test_object_alt.three)
 
+    def test_override_inherited(self):
+        """Overriding inherited declarations"""
+        class TestObjectFactory(factory.Factory):
+            class Meta:
+                model = TestObject
+
+            one = 'one'
+
+        class TestObjectFactory2(TestObjectFactory):
+            one = 'two'
+
+        test_object = TestObjectFactory2.build()
+        self.assertEqual('two', test_object.one)
+
+    def test_override_inherited_deep(self):
+        """Overriding inherited declarations"""
+        class TestObjectFactory(factory.Factory):
+            class Meta:
+                model = TestObject
+
+            one = 'one'
+
+        class TestObjectFactory2(TestObjectFactory):
+            one = 'two'
+
+        class TestObjectFactory3(TestObjectFactory2):
+            pass
+
+        test_object = TestObjectFactory3.build()
+        self.assertEqual('two', test_object.one)
+
     def test_inheritance_and_sequences(self):
         """Sequence counters should be kept within an inheritance chain."""
         class TestObjectFactory(factory.Factory):
@@ -1459,12 +1493,9 @@ class BetterFakeModelManager(object):
         instance.id = 2
         return instance, True
 
-    def values_list(self, *args, **kwargs):
+    def using(self, db):
         return self
 
-    def order_by(self, *args, **kwargs):
-        return [1]
-
 
 class BetterFakeModel(object):
     @classmethod
@@ -1587,14 +1618,14 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
         o1 = TestModelFactory()
         o2 = TestModelFactory()
 
-        self.assertEqual('foo_2', o1.a)
-        self.assertEqual('foo_3', o2.a)
+        self.assertEqual('foo_0', o1.a)
+        self.assertEqual('foo_1', o2.a)
 
         o3 = TestModelFactory.build()
         o4 = TestModelFactory.build()
 
-        self.assertEqual('foo_4', o3.a)
-        self.assertEqual('foo_5', o4.a)
+        self.assertEqual('foo_2', o3.a)
+        self.assertEqual('foo_3', o4.a)
 
     def test_no_get_or_create(self):
         class TestModelFactory(factory.django.DjangoModelFactory):
@@ -1605,7 +1636,7 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
 
         o = TestModelFactory()
         self.assertEqual(None, o._defaults)
-        self.assertEqual('foo_2', o.a)
+        self.assertEqual('foo_0', o.a)
         self.assertEqual(2, o.id)
 
     def test_get_or_create(self):
@@ -1621,7 +1652,7 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
 
         o = TestModelFactory()
         self.assertEqual({'c': 3, 'd': 4}, o._defaults)
-        self.assertEqual('foo_2', o.a)
+        self.assertEqual('foo_0', o.a)
         self.assertEqual(2, o.b)
         self.assertEqual(3, o.c)
         self.assertEqual(4, o.d)
@@ -1641,7 +1672,7 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
 
         o = TestModelFactory()
         self.assertEqual({}, o._defaults)
-        self.assertEqual('foo_2', o.a)
+        self.assertEqual('foo_0', o.a)
         self.assertEqual(2, o.b)
         self.assertEqual(3, o.c)
         self.assertEqual(4, o.d)
@@ -1773,7 +1804,7 @@ class PostGenerationTestCase(unittest.TestCase):
                 model = TestObject
             one = 3
             two = 2
-            three = factory.RelatedFactory(TestRelatedObjectFactory, name='obj')
+            three = factory.RelatedFactory(TestRelatedObjectFactory, 'obj')
 
         obj = TestObjectFactory.build()
         # Normal fields
@@ -1907,6 +1938,23 @@ class CircularTestCase(unittest.TestCase):
         self.assertIsNone(b.foo.bar.foo.bar)
 
 
+class SelfReferentialTests(unittest.TestCase):
+    def test_no_parent(self):
+        from .cyclic import self_ref
+
+        obj = self_ref.TreeElementFactory(parent=None)
+        self.assertIsNone(obj.parent)
+
+    def test_deep(self):
+        from .cyclic import self_ref
+
+        obj = self_ref.TreeElementFactory(parent__parent__parent__parent=None)
+        self.assertIsNotNone(obj.parent)
+        self.assertIsNotNone(obj.parent.parent)
+        self.assertIsNotNone(obj.parent.parent.parent)
+        self.assertIsNone(obj.parent.parent.parent.parent)
+
+
 class DictTestCase(unittest.TestCase):
     def test_empty_dict(self):
         class TestObjectFactory(factory.Factory):
